#!/usr/bin/env python

#######################################################
#
# Manages database files from cddb.com and freedb.org
#
#######################################################

import time
import string
import re 
import os
import sys

#######################################################


#######################################################
## 
#######################################################

patternDiscLength = re.compile(r'# Disc length:[ \t]+([0-9]+).*$')
patternTrackOffsets = re.compile(r'# Track frame offsets:[ \t\r]*$')
patternOffset = re.compile(r'#[ \t]+([0-9]+)[ \t\r]*$')
patternTocComment = re.compile(r'^[#\.](.*)$')
patternTocEmpty = re.compile(r'^[ \t\r\n]*$')
patternTocTrackTitle = re.compile(r'^TTITLE([0-9]+)\=(.*?)[\r]*$')
patternTocTrackExt = re.compile(r'^EXTT([0-9]+)\=(.*?)[\r]*$')
patternTocDiscTitle = re.compile(r'^DTITLE\=(.*?)[\r]*$')
patternTocDiscYear = re.compile(r'^DYEAR\=(.*?)[\r]*$')
patternTocDiscGenre = re.compile(r'^DGENRE\=(.*?)[\r]*$')
patternTocDiscId = re.compile(r'^DISCID\=(.*?)[\r]*$')
patternTocDiscExt = re.compile(r'^EXTD\=(.*?)[\r]*$')
patternTocPlayOrder = re.compile(r'^PLAYORDER\=(.*)$')

PatternFileNameWithLeadingDigits = re.compile(r'^([0-9]+)[^0-9].*$')
PatternFileNameWithLeadingDigits2 = re.compile(r'^track([0-9]+)[^0-9].*$')


class Toc:
    
    def __init__(self, filename):

        input = open(filename, "r")
        lines = input.readlines()
        input.close()

        self._discid = ""
        self._disctitle = ""
        self._discyear = ""
        self._discgenre = ""
        self._discext = ""
        self._artist = ""
        self._tracktitle= {}
        self._trackext = {}
        self._tracktimes = []
        self._disctime = None

        while lines:
            line = lines[0]
            lines = lines[1:]

            match = patternTrackOffsets.search(line)
            if match:
                while 1:
                    line = lines[0]
                    lines = lines[1:]
                    match = patternOffset.search(line)
                    if match:
                        self._tracktimes.append( int(match.group(1)))
                    else:
                        break
                continue

            match = patternDiscLength.search(line)
            if match:
                self._disctime = int(match.group(1))

                self._tracktimes.append( self._disctime * 75 )
                for i in range(len(self._tracktimes)-1):
                     self._tracktimes[i] = (self._tracktimes[i+1]-self._tracktimes[i])/75
                continue


            if line[0] == '#' or line[0] == ' ' or line[0] == '\n' or line[0] == '\r' or line[0] == '\t':
                continue

            match = patternTocPlayOrder.search(line)
            if match:
                continue
            match = patternTocDiscExt.search(line)
            if match:
                self._discext =   self._discext + match.group(1)
                continue
            match = patternTocDiscId.search(line)
            if match:
                self._discid =   self._discid + match.group(1)
                continue
            match = patternTocDiscGenre.search(line)
            if match != None:
                self._discgenre =   self._discgenre + match.group(1)
                continue
            match = patternTocDiscYear.search(line)
            if match != None:
                self._discyear =   self._discyear + match.group(1)
                continue
            match = patternTocDiscTitle.search(line)
            if match != None:
                self._disctitle =   self._disctitle + match.group(1)
                self._artist = self._disctitle
                continue
            match = patternTocTrackTitle.search(line)
            if match != None:
                track =  int(match.group(1))
                if self._tracktitle.has_key(track):
                    self._tracktitle[track] = self._tracktitle[track] +  match.group(2)
                else:
                    self._tracktitle[track] = match.group(2)
                continue    
            match = patternTocTrackExt.search(line)
            if match != None:
                track =  int(match.group(1))
                if self._trackext.has_key(track):
                    self._trackext[track] = self._trackext[track] + match.group(2)
                else:
                    self._trackext[track] =  match.group(2)
                continue
            file = open("xxx","wa")
            file.write(line)
            file.close()
            print "BAD LINE:" + line
            assert(0)

    def title_get(self):
        return self._disctitle

    def track_get(self,string):
        dummy, file = os.path.split(string)
        match = PatternFileNameWithLeadingDigits.search(file)
        if match == None:
            match = PatternFileNameWithLeadingDigits2.search(file)
        if match == None:
            return (None,0,0)
        index = int(match.group(1)) - 1
        if index < len(self._tracktitle) and index >= 0:
            return (self._tracktitle[index],self._trackext[index],self._tracktimes[index])
        else:
            return (None,0,0)

    def artist_get(self):
        return self._artist
    
    def dump(self):
        print "DISCID:    " + self._discid
        print "DISCTITLE: " + self._disctitle
        print "DISKEXT:   " + self._discext
        print "TRACK TITLES"
        for k in self._tracktitle.keys():
            print "%2d" % k,self._tracktitle[k]
        print "TRACK EXTS"
        for k in self._trackext.keys():
            print "%2d" % k,self._trackext[k]
        print "TRACK TIMES"
        for k in self._tracktimes:
            print k
            
#######################################################
#######################################################


class TocContainer:
    def __init__(self):
        self._container = {}

    def find_toc(self,dirname):
        filename = os.path.join(dirname, "toc")
        if self._container.has_key(filename):
            return self._container[filename]

        if not os.access(filename,os.R_OK):
            self._container[filename] = None
            return None

        toc = Toc(filename)
        self._container[filename] = toc
        return toc
             
#######################################################
#######################################################

if __name__ == '__main__':
   
      for i in sys.argv[1:]:
        print i
        if not os.access(i,os.R_OK):
            print "cannot open " + i
            continue;
        toc = Toc(i)
        toc.dump()

#######################################################
## eof
#######################################################







