#!/usr/bin/env python2.1
# -*- python -*-
#
#      Copyright (C) 2002 Robert Muth <robert@muth.org>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; version 2 of June 1991.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program in form of the file COPYING; 
#      if not, write to the 
#
#      Free Software Foundation, Inc. <http://www.fsf.org>
#      59 Temple Place, Suite 330, 
#      Boston, MA 02111-1307  USA
#
#######################################################
#######################################################

import re
import os
import sys
import popen2

#######################################################
## 
#######################################################
logfile = None

def Message(prefix,message):
    global logfile
    if not logfile:
        logfile = open("musicus.log","wa")
    logfile.write(prefix + ": " + message)

def Debug(message):
    Message("D",message)

def Info(message):
    Message("I",message)

def Warning(message):
    Message("W",message)

def Error(message):
    Message("E",message)
    sys.exit(-1)
    
#######################################################
## 
#######################################################

def tokenize(line):
#    print line
    token = []
    whitespace = ' \t\r\n'
    quotes = '\'"'
    state = 'n'
    curr = ""
    for char in line:
        if char in whitespace:
            if state in quotes:
                curr = curr + char
            elif curr != "":
                token.append(curr)
                curr = ""
            continue
        
        if char in quotes:
            if state == 'n':
                if curr != "":
                    raise "whitespace missing"
                curr = curr + char
                state = char                
            elif state == char:
                curr = curr + char
                token.append(curr)
                curr = ""
                state = 'n'
                
            else:
                curr = curr + char

            continue

        curr = curr + char

    if state == 'q':
        raise "quote missing"

    if curr != "":
        token.append(curr)
      
    return token

#######################################################
#
#######################################################

def process_result(result):
    lastline = result[-1]
    token = tokenize(lastline)
    if token[0] != "@":
        raise "bad token"
    if token[1] == "ERROR":
        token[1] = 1
        return token[1:]
    if token[1] == "OK":
        token[1] = 0
        return token[1:]
    raise "bad token"
        
#######################################################
#######################################################
    
class Musicus:
    """
    This class handles the communication with the
    musicus program
    """
    def __init__(self):
#        self._process = popen2.Popen4("musicus -output libALSA.so",0)
        self._process = popen2.Popen4("musicus",0)
        self._stdin = self._process.tochild
        self._stdout = self._process.fromchild
        
#        self._stdin, self._stdout = os.popen2("musicus","t",0)
#        print self._stdin
#        print self._stdout

    def kill(self):
        os.kill( self._process.pid,9)
#        self._stdin.close()
#        self._stdout.close()
        return
    
    def issue_commad(self,line):
        try:
            self._stdin.write(line)
            result = []
            while 1:
                line = self._stdout.readline()
                result.append(line)
                if line[0] == '@':
                    break
            return result
        except IOError:
            raise "\nERROR\npipe to musicus player broken - was musicus installed?"
            
    def setvol(self,lvol,rvol):
        result = self.issue_commad("setvol %d %d\n" % (lvol,rvol))
        return process_result( result )

    def getvol(self):
        result = self.issue_commad("getvol\n")
        return process_result( result )
    
    def play(self,song):
        result = self.issue_commad("play %s\n" % song)
        return process_result( result )

    def stop(self):
        result = self.issue_commad("stop\n")
        return process_result( result )

    def pause(self):
        result = self.issue_commad("pause\n")
        return process_result( result )

    def unpause(self):
        result = self.issue_commad("unpause\n")
        return process_result( result )

    def getlength(self):
        result = self.issue_commad("getlength\n")
        return process_result( result )

    def getpos(self):
        result = self.issue_commad("getpos\n")
        return process_result( result )

    def setpos(self,pos):
        result = self.issue_commad("setpos %d\n" % pos)
        return process_result( result )


#######################################################
#######################################################
            
if __name__ == '__main__':
    musicus = Musicus()
    for i in sys.argv[1:]:
        print i
        result = musicus.getvol()
        print result

        result = musicus.setvol(99,99)
        print result
    
        result = musicus.play(i)
        print result

        result = musicus.getpos()
        print result

        result = musicus.getpos()
        print result

        result = musicus.stop()
        print result

        
    while 1:
        line = sys.stdin.readline()
        result = musicus.issue_commad(line)

        print result
