#!/usr/bin/env python

#######################################################
#######################################################


import fcntl
import os
import re
import sys
import string
import signal
import time

#######################################################
#######################################################

def which(program):
    for path in string.split(os.environ["PATH"], ":"):
        if os.path.exists(os.path.join(path, program)):
            return os.path.join(path, program)


class PlayingSong:

    def __init__(self, player, filename, offset, soundcard):
        self._offset = offset
        self._player = player
        self._soundcard = soundcard
        self._filename = filename
        self._argv = player.prepare_commandline(filename,offset,soundcard)
        self._null = os.open("/dev/null",0)
        
#        self._stdout_r, self._stdout_w = os.pipe()
#        self._stderr_r, self._stderr_w = os.pipe()

        self._pid = os.fork()
        self._result = (-1,-1)
        
        if self._pid == 0:
            signal.signal(signal.SIGTERM, signal.SIG_DFL)
#            os.dup2(self._stdout_w, sys.stdout.fileno())
#            os.dup2(self._stderr_w, sys.stderr.fileno())
            os.dup2(self._null, sys.stdout.fileno())
            os.dup2(self._null, sys.stderr.fileno())
            os.setpgrp()
            try: os.execv(self._argv[0], self._argv)
            except: time.sleep(1); os._exit(1)
            
        self._paused = 0
        self._seek_step = None
        self._time_done = 0
        self._time_left = 0
        self._is_killed = 0
        
    def is_paused(self):
        return self._paused

    def pause(self):
        self._paused = 1
        try: os.kill(-self._pid, signal.SIGSTOP)
        except os.error: pass

    def unpause(self):
        #assert not self.is_stopped()
        self._paused = 0
        try: os.kill(-self._pid, signal.SIGCONT)
        except os.error: pass


    def parse_stdout(self):
        os.read(self.stdout_r, 256)

    def parse_stderr(self):
        os.read(self.stderr_r, 256)


    def kill(self):
        self.is_paused() and self.unpause()
        os.kill(self._pid, signal.SIGTERM)
#        try:
#            while 1:
#                try: os.kill(self._pid, signal.SIGTERM)
#                except os.error: pass
#                os.waitpid(self._pid, os.WNOHANG)
#        except Exception: pass

    def is_killed(self):
        if not self._is_killed:
            (pid,signal) = self._result = os.waitpid(self._pid, os.WNOHANG)
            #print pid,signal
            # 19 = signal.SIGSTOP ????
            self._is_killed = (pid == self._pid) and (signal != 19)
        return self._is_killed    

    def current_relative_position():
        return None

    def current_time():
        return None

    def total_time():
        return None

#######################################################
#######################################################

class Player:
    
    def __init__(self, name,commandline, extension, has_progress):
        self._name = name
        self._commandline = commandline
        self._extensions = extension
        self._has_progress = has_progress

    def name(self):
        return self._name

    def extensions(self):
        return self._extensions
    
    def prepare_commandline(self, filename, offset, soundcard):
        argv = string.split(self._commandline)
        argv[0] = which(argv[0])
        for i in range(len(argv)):
            if argv[i] == '%s': argv[i] = filename
            if argv[i] == '%d': argv[i] = str(offset)
            if argv[i] == '%a': argv[i] = soundcard.device_dsp_get()
        return argv

    def play_song(self,filename,offset,soundcard):
        return PlayingSong( self, filename, offset, soundcard )

    def help_string(self):
         argv = string.split(self._commandline)
         return ("%-15s" % argv[0]) + string.join(self._extensions,",")
         
    
#######################################################
#######################################################


class AvailablePlayers:
    def __init__(self):
        self._players = []

    def register_player(self,player):
        for e in player.extensions():
            for p in self._players:
                for f in p.extensions():
                    assert e != f
        self._players.append(player)            
          

    def find_player(self,filename):
        (head,extension) = os.path.splitext(filename)
        for p in  self._players:
            for f in p.extensions():
                if extension == f:
                    return p
        return None

    def help_strings(self):
        messages = []
        for p in self._players:
            messages.append(p.help_string())
        return messages
#######################################################
#######################################################

class Soundcard:
    def __init__(self):
        for self.device_dsp in ["/dev/sound/dsp", "/dev/dsp"]:
            if os.path.exists(self.device_dsp): break

        for self.device_mixer in ["/dev/sound/mixer", "/dev/mixer"]:
            if os.path.exists(self.device_mixer): break

    def volume_lo(self):
        return 0

    def volume_hi(self):
        return 100
        
    def volume_inc(self):
        self.volume_set( self.volume_get()+1) 

    def volume_dec(self):
        self.volume_set( self.volume_get()+1) 

    def volume_get(self):
        fd = os.open(self.device_mixer, os.O_RDONLY)
        volume = ord(fcntl.ioctl(fd, 0x80044d00, "."))
        os.close(fd)
        return volume

    def volume_set(self, v):
        if v >= self.volume_lo() and v <= self.volume_hi():
            fd = os.open(self.device_mixer, os.O_WRONLY)
            fcntl.ioctl(fd, 0xc0044d00, "%c%c" % (v, v))
            os.close(fd)

    def device_mixer_get(self):
       return self.device_mixer

    def device_dsp_get(self):
       return self.device_dsp
   
if __name__ == '__main__':
    soundcard = Soundcard()
    print "volume",soundcard.volume_get()
    
    players = AvailablePlayers()

    players.register_player( Player("mpg","madplay -q --no-tty-control -s %d %s", [".mp1",".mp2",".mp3"], 1) )
    players.register_player( Player("ogg","ogg123 -q -v -d oss -k %d %s", [".ogg"],1) )
    players.register_player( Player("sox","sox %s -t ossdsp %a", [".aiff",".au",".cdr",".wav",".WAV"],1) )
    players.register_player( Player("mpc","mppdec %s /dev/dsp", [".mp+",".mpc"],0) )
#    players.register_player( Player("xxx","splay -f %s",["mp3"],1) )
#    players.register_player( Player("mpg","mpg123 -b 2000 -q -v -k %d %s", [".mp1",".mp2",".mp3"], 1) )


    for i in sys.argv[1:]:
        print i
        player = players.find_player(i)
        if player != None:
            print "playing ",player.name()
            song = player.play_song(i,0,soundcard)
            print song._pid

            while not song.is_killed():
                time.sleep(1)
        else:
            print "skipping"
